@@ -30,16 +30,18 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <xf86drm.h>
 #include <string.h>
 #include <unistd.h>
-
 #include <sys/stat.h>
 
+#include "libdrm_macros.h"
 #include "internal.h"
 
 #define PATH_SIZE 512
@@ -68,7 +70,7 @@ linux_name_from_sysfs(int fd, char **out)
 
 	ret = fstat(fd, &buffer);
 	if (ret)
-		return ret;
+		return -EINVAL;
 
 	if (!S_ISCHR(buffer.st_mode))
 		return -EINVAL;
@@ -101,21 +103,31 @@ linux_from_sysfs(int fd, struct kms_driver **out)
 	if (ret)
 		return ret;
 
+#ifdef HAVE_INTEL
 	if (!strcmp(name, "intel"))
 		ret = intel_create(fd, out);
+	else
+#endif
 #ifdef HAVE_VMWGFX
-	else if (!strcmp(name, "vmwgfx"))
+	if (!strcmp(name, "vmwgfx"))
 		ret = vmwgfx_create(fd, out);
+	else
 #endif
 #ifdef HAVE_NOUVEAU
-	else if (!strcmp(name, "nouveau"))
+	if (!strcmp(name, "nouveau"))
 		ret = nouveau_create(fd, out);
+	else
 #endif
 #ifdef HAVE_RADEON
-	else if (!strcmp(name, "radeon"))
+	if (!strcmp(name, "radeon"))
 		ret = radeon_create(fd, out);
-#endif
 	else
+#endif
+#ifdef HAVE_EXYNOS
+	if (!strcmp(name, "exynos"))
+		ret = exynos_create(fd, out);
+	else
+#endif
 		ret = -ENOSYS;
 
 	free(name);
@@ -213,9 +225,12 @@ linux_from_udev(int fd, struct kms_driver **out)
 }
 #endif
 
-int
+drm_private int
 linux_create(int fd, struct kms_driver **out)
 {
+	if (!dumb_create(fd, out))
+		return 0;
+
 	if (!linux_from_udev(fd, out))
 		return 0;
 
